(define-module (elmord utils ports)
  #:use-module (oop goops)
  #:use-module (elmord utils sequences)
  #:use-module (srfi srfi-1)  ;; list library
  #:use-module (srfi srfi-41) ;; streams
  #:use-module (ice-9 rdelim)
  #:use-module (ice-9 popen)
  #:export (resource->stream file->stream pipe->stream *eof-object*)
  )

(define* (resource->stream #:key resource consume end? close)
  (stream-let loop ()
    (let ([item (catch #t
                  (lambda ()
                    (consume resource))
                  (lambda err
                    (close resource)
                    (apply throw err)))])
      (cond [(end? item)
             (close resource)
             stream-null]
            [else (stream-cons item (loop))]))))
  

(define* (file->stream pathname #:optional [consume read-line])
  (resource->stream #:resource (open-input-file pathname)
                    #:consume consume
                    #:end? eof-object?
                    #:close close-input-port))

(define* (pipe->stream command #:optional [consume read-line])
  (resource->stream #:resource (if (string? command)
                                   (open-pipe command OPEN_READ)
                                 (apply open-pipe* OPEN_READ command))
                    #:consume consume
                    #:end? eof-object?
                    #:close close-pipe))

(define *eof-object*
  (with-input-from-string "" read))
